# $Id: ModEnforce.pm,v 1.8 2004-05-16 23:27:26 uhtu Exp $
# Copyright (c) 2002-2004 Robert Hagemann <bobzilla@gothictech.com>
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

package ModEnforce;

@ModEnforce::ISA = qw(OmniMod);

use strict;
use Digest::MD5 qw(md5 md5_hex md5_base64);

my $passFile = 'passwd';
my $on = 1;
my $errorstring = "";

sub name() {
  my $this = shift;
  return "enforcer";
}

sub help {
  return if ! $on;

  if ( defined wantarray ) { # do we have help?
    return 1;
  }

  my $this = shift;
  my $irc = shift;
  my $nick = shift;

  $irc->notice( $nick, "setpasswd <password> [<oldpassword>]" );
  $irc->notice( $nick, "op <password>" );
  $irc->notice( $nick, "(Your username and host must be already be in the server's passwd file for that channel)" );
}

sub handlePrivateMessage {
  my $this = shift;
  my $irc = shift;
  my $nick = shift;
  my $message = shift;
  my $event = shift;
  my $conn = shift;

  return if ! $on;
  return if ! &applicable( $message );

  if ( $message =~ /^op\s+(.+?)\s*$/ ) {
    my $password = $1;
    my @channels = ();
    if ( &checkPassword( $event->user, $event->host, $password, \@channels ) ) {
      foreach my $channel ( @channels ) {
        $conn->mode( $channel, '+o', $nick );
      }
    }
    else {
      $irc->privmsg( $nick, "Error checking password: $errorstring" );
    }
  }
  elsif ( $message =~ /^setpasswd\s+(.+?)\s+(.+?)\s*$/ 
      or $message =~ /^setpasswd\s+(.+?)\s*$/ ) {
    my $password = $1;
    my $oldpassword = $2;
    if ( &setPassword( $event->user, $event->host, $password, $oldpassword ) ) {
      $irc->privmsg( $nick, "New password set" );
    }
    else {
      $irc->privmsg( $nick, "Error setting password: $errorstring" );
    }
  }
  elsif ( $message =~ /^adminsetpasswd\s+(.+?)\s+(.+?)\s+(.+?)\s*$/ ) {
    my $user = $1;
    my $password = $2;
    my $adminpassword = $3;
    
    if ( &setAdminPassword( $event->user, $event->host, $user, $password, 
        $adminpassword ) ) {
      $irc->privmsg( $nick, "New password set for $user" );
    }
    else {
      $irc->privmsg( $nick, "Error setting password: $errorstring" );
    }
  }
  else {
    $this->help( $irc, $nick );
  }
}

sub applicable {
  my $message = shift;
  if ( $message =~ /^(op|setpasswd|adminsetpasswd)\s+/ ) {
    return 1;
  }
  return 0;
}

sub checkPassword {
  my $user = shift;
  my $host = shift;
  my $password = shift;
  my $channels = shift;
  
  $user =~ s/^~//sgi;

  return 0 if ( ! &checkPasswdFile( $user, $host, $channels ) );

  my $passhash = OmniHash::getHash( "passwords" );
  if ( ! defined $$passhash{ $user } ) {
    OmniHash::closeAllHashes();
    $errorstring = "No password for user: '$user'";
    return 0;
  }
  if ( $$passhash{ $user } eq md5_base64( $password ) ) {
    OmniHash::closeAllHashes();
    return 1;
  }

  OmniHash::closeAllHashes();
  $errorstring = "Passwords do not match";
  # print "user: $user storedhash: " . $$passhash{ $user } . " newhash: " . md5_base64( $password ) . "\n";
  return 0;
}

sub setPassword {
  my $user = shift;
  my $host = shift;
  my $password = shift;
  my $oldpassword = shift;

  $user =~ s/^~//sgi;

  return 0 if ( ! &checkPasswdFile( $user, $host, undef ) );

  my $passhash = OmniHash::getHash( "passwords" );
  
  if ( defined $$passhash{ $user } 
      and $$passhash{ $user } !~ /^\s*$/ ) {
    OmniHash::closeAllHashes();
    # there is an old password set, make sure the old passwords match
    if ( ! defined $oldpassword
        or ! &checkPassword( $user, $host, $oldpassword, undef ) ) {
      $errorstring = "Old password does not match";
      return 0;
    }
  }

  # checkPassword may have closed hashes on us
  OmniHash::closeAllHashes();
  $passhash = OmniHash::getHash( "passwords" );
  $$passhash{ $user } = md5_base64( $password );
  OmniHash::closeAllHashes();

  return 1;
}

sub setAdminPassword {
  my $adminuser = shift;
  my $adminhost = shift;
  my $user = shift;
  my $password = shift;
  my $adminpassword = shift;
  
  # HACK should be in config
  if ( $adminuser !~ 'bobzilla' ) {
    $errorstring = "User not an admin";
    return 0;
  }
  
  return 0 
    if ( ! &checkPassword( $adminuser, $adminhost, $adminpassword, undef ) );

  my $passhash = OmniHash::getHash( "passwords" );
  # print "pass:$password user:$user adminuser: $adminuser adminpass:$adminpassword\n";
  $$passhash{ $user } = md5_base64( $password );
  OmniHash::closeAllHashes();

  return 1;
}

sub checkPasswdFile {
  my $user = shift;
  my $host = shift;
  my $channels = shift;
  my $retval = 0;

  $user =~ s/^~//sgi;
  
  open PASSWD, $passFile;
  while ( <PASSWD> ) {
    my $line = $_;
    next if $line =~ /^\s*#/si;
    
    my ($fileuser, $filehost, $filechannel) = split /:/, $line;
    next if ! defined $fileuser;
    next if ! defined $filehost;
    next if ! defined $filechannel;
    $filehost =~ s/\./\\./sgi;
    $filechannel =~ s/\s*$//sgi;

    if ( $fileuser =~ /^$user$/si ) {
      if ( $host =~ /$filehost$/si ) {
        push @$channels, $filechannel if defined $channels;
        $retval = 1;
      }
    }
  }
  close PASSWD;
  
  if ( ! $retval ) {
    $errorstring = "User '$user\@$host' not in config file";
  }
  return $retval;
}

1;
